'''
    Mapseg, a program for segmenting maps and producing OSM polygons
    Copyright (C) 2010 Tim Sheerman-Chase

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 or version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
import pickle
import math
import numpy as np

def OrthoPolygon(polyIn):
	#print poly
	poly = []
	poly[:] = polyIn['path']
	poly.pop(-1)
	meta = polyIn['meta'].copy()

	#Get average direction in sector -pi/4 to +pi/4
	angSum = 0.0
	angCount = 0
	angList = []
	for i in range(len(poly)):
		pt1 = poly[i]
		pt2 = poly[(i+1)%len(poly)]
		dx = pt1[0] - pt2[0]
		dy = pt1[1] - pt2[1]
		ang = math.atan2(dy,dx)
		while ang > math.pi/4.0:
			ang = ang - math.pi/2.0
		while ang < -math.pi/4.0:
			ang = ang + math.pi/2.0
		angSum = angSum + ang
		angCount = angCount + 1
		angList.append(ang)
		#print dx,dy,ang
	
	avAng = angSum / angCount
	x = np.array(angList)
	var1 = np.mean( (x-np.mean(x)) ** 2 )
	#print "av1",avAng,var1

	#Get average direction in sector 0 to +pi/2
	angSum = 0.0
	angCount = 0
	angList = []
	for i in range(len(poly)):
		pt1 = poly[i]
		pt2 = poly[(i+1)%len(poly)]
		dx = pt1[0] - pt2[0]
		dy = pt1[1] - pt2[1]
		ang = math.atan2(dy,dx)
		while ang > math.pi/2.0:
			ang = ang - math.pi/2.0
		while ang < 0.0:
			ang = ang + math.pi/2.0
		angSum = angSum + ang
		angCount = angCount + 1
		angList.append(ang)
		#print dx,dy,ang
	
	avAng2 = angSum / angCount
	x = np.array(angList)
	var2 = np.mean( (x-np.mean(x)) ** 2 )
	while avAng2 > math.pi/4.0:
		avAng2 = avAng2 - math.pi/2.0
	
	#print "av2",avAng2,var2

	#Use the fit with the lowest variance (closest fit)
	minVar = var1
	if var2 < var1:
		avAng = avAng2
		minVar = var2

	if minVar > 0.1:
		meta['mapseg:not_orthogonal'] = 'FIXME'

	vert = (math.cos(avAng), math.sin(avAng))
	hori = (math.sin(avAng), -math.cos(avAng))

	#Rotate to align roughly along axis
	npol = []
	for pt in poly:
		npt = [pt[0] * vert[0] + pt[1] * vert[1],pt[0] * hori[0] + pt[1] * hori[1]]
		#print "rot",pt, npt
		npol.append(npt)

	#Align segments with axis
	prevDirection = None
	for i in range(len(npol)+1):
		pt1 = npol[i%len(npol)]
		pt2 = npol[(i+1)%len(npol)]		
		dx = abs(pt1[0] - pt2[0])
		dy = abs(pt1[1] - pt2[1])
		#print dx, dy
		direction = None
		if dx > dy:
			#print "horizontal"
			av = 0.5*(pt1[1] + pt2[1])
			pt1[1] = av
			pt2[1] = av
			direction = 'h'
		else:		
			#print "vertical"
			av = 0.5*(pt1[0] + pt2[0])
			pt1[0] = av
			pt2[0] = av
			direction = 'v'
		if direction == prevDirection:
			meta['mapseg:extra_node'] = 'FIXME'
		prevDirection = direction

	#print "npol",npol	

	#Rotate back
	out = []
	for pt in npol:
		npt = [pt[0] * vert[0] + pt[1] * vert[1],pt[0] * hori[0] + pt[1] * hori[1]]
		#print "rot2",pt, npt
		out.append(npt)	

	out.append(out[0])

	polyOut = {}
	polyOut['path'] = out
	polyOut['meta'] = meta
	return polyOut

def OrthoPolygons(polys):
	out = []
	#polys = [[(416, 419), (419, 427), (409, 430), (406, 422), (416, 419)]]
	count = 0
	for poly in polys:
		orthoPoly = {}
		orthoPoly = OrthoPolygon(poly)
		out.append(orthoPoly)
		#print poly,orthoPoly
		count = count + 1
	return out

if __name__ == '__main__':
	#fi = open("polydump.txt","rt")
	#polys = pickle.load(fi)
	#OthoPolygons(polys)

	#x = ([[1.0,2.0,3.0,4.0,6.0],[1.0,2.0,3.0,4.0,5.0]])
	#x = ([[0.0,0.1],[2.0,0.0],[2.0,1.0],[0.0,1.0]])
	#x = ([[0.0,0.0],[2.0,-1.0],[4.0,4.0],[2.0,5.0]])
	#x =[(416, 419), (419, 427), (409, 430), (406, 422), (416, 419)]
	x = [(0,0),(1.01,1),(0,1.99),(-1,1),(0,0)]
	ortho = OrthoPolygon(x)
	print ortho
	for i in range(len(ortho)):
		j = (i+1) % len(ortho)
		k = (i+2) % len(ortho)
		dx1 = ortho[i][0] - ortho[j][0]
		dy1 = ortho[i][1] - ortho[j][1]
		dx2 = ortho[j][0] - ortho[k][0]
		dy2 = ortho[j][1] - ortho[k][1]
		print "dot check",dx1 * dx2 + dy1 * dy2

